package fr.gasser.gui.editor;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.swing.*;
import javax.swing.text.*;
import java.awt.*;
import java.awt.event.ItemEvent;

public class App extends JFrame {

    private final static Logger LOGGER = LoggerFactory.getLogger(App.class);
    private static final Font DEFAULT_FONT = new Font("Arial", Font.PLAIN, 14);
    private static final Color DEFAULT_COLOR = Color.BLACK;
    private static final String LOREM_IPSUM = "Quisque eget mattis nulla. Praesent efficitur fermentum neque quis fringilla. Quisque laoreet eros maximus nibh eleifend, eu faucibus tortor egestas. Proin finibus eu tortor eget dictum. Interdum et malesuada fames ac ante ipsum primis in faucibus. Nullam quis nisi sit amet magna ullamcorper sollicitudin. Curabitur vel pellentesque lorem, in tristique dolor.";

    private JTextPane editArea;
    private JButton colorButton;

    private App() throws HeadlessException {
        setDefaultCloseOperation(EXIT_ON_CLOSE);
        try {
            UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
        } catch (Exception e) {
            LOGGER.warn(e.getMessage());
        }
        initComponents();
        pack();
        setVisible(true);
    }

    private void initComponents() {
        setLayout(new BorderLayout());
        final JPanel topPane = new JPanel();
        topPane.setLayout(new BorderLayout());
        final JPanel bottomPane = new JPanel();

        editArea = new JTextPane();
        editArea.setText(LOREM_IPSUM);
        editArea.setFont(DEFAULT_FONT);
        final JScrollPane textScrollPane = new JScrollPane(editArea);
        textScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        textScrollPane.setPreferredSize(new Dimension(144, 256));
        topPane.add(textScrollPane);

        final FontChooser fontChooser = new FontChooser();
        fontChooser.addItemListener(this::doChangeFont);
        fontChooser.setSelectedItem(fontChooser.get(DEFAULT_FONT));
        bottomPane.add(fontChooser);

        final JComboBox<Integer> fontSizeChooser = new JComboBox<>();
        for (int i=5; i < 200; i++) fontSizeChooser.addItem(i);
        fontSizeChooser.addItemListener(this::doChangeFontSize);
        fontSizeChooser.setSelectedItem(DEFAULT_FONT.getSize());
        bottomPane.add(fontSizeChooser);

        colorButton = new JButton();
        colorButton.setHorizontalAlignment(SwingConstants.CENTER);
        colorButton.setVerticalAlignment(SwingConstants.CENTER);
        colorButton.setIcon(new SquareIcon(30, DEFAULT_COLOR));
        colorButton.addActionListener(evt -> doChangeColor());
        bottomPane.add(colorButton);

        final JButton boldButton = new JButton(new StyledEditorKit.BoldAction());
        boldButton.setText("B");
        bottomPane.add(boldButton);

        final JButton italicButton = new JButton(new StyledEditorKit.ItalicAction());
        italicButton.setText("I");
        bottomPane.add(italicButton);

        final JButton underlineButton = new JButton(new StyledEditorKit.UnderlineAction());
        underlineButton.setText("U");
        bottomPane.add(underlineButton);

        final Container container = getContentPane();
        container.add(topPane, BorderLayout.NORTH);
        container.add(bottomPane, BorderLayout.SOUTH);
    }

    private void doChangeColor() {
        final Color c = JColorChooser.showDialog(this, "Choose a color", Color.BLACK);
        SimpleAttributeSet attr = new SimpleAttributeSet(getSelectedElement().getAttributes().copyAttributes());
        StyleConstants.setForeground(attr, c);
        editArea.setCharacterAttributes(attr, true);
        colorButton.setIcon(new SquareIcon(30, c));

    }

    private void doChangeFont(ItemEvent evt) {
        final Font f = (Font) evt.getItem();

        SimpleAttributeSet attr = new SimpleAttributeSet(getSelectedElement().getAttributes().copyAttributes());
        StyleConstants.setFontFamily(attr, f.getName());
        editArea.setCharacterAttributes(attr, true);
    }

    private void doChangeFontSize(ItemEvent evt) {
        SimpleAttributeSet attr = new SimpleAttributeSet(getSelectedElement().getAttributes().copyAttributes());
        StyleConstants.setFontSize(attr, (int) evt.getItem());
        editArea.setCharacterAttributes(attr, true);
    }

    private  Element getSelectedElement() {
        final StyledDocument doc = (StyledDocument) editArea.getDocument();
        return doc.getCharacterElement(editArea.getSelectionStart());
    }

    public static void main(String[] args) {
        EventQueue.invokeLater(App::new);
    }

    private static class SquareIcon implements Icon {
        private final int size;
        private final Color color;

        SquareIcon(int size, Color color) {
            this.size = size;
            this.color = color;
        }

        @Override
        public void paintIcon(Component component, Graphics graphics, int i, int i1) {
            final Point center = new Point(component.getWidth()/4, component.getHeight()/4);
            final Graphics2D g2d = (Graphics2D) graphics;
            g2d.setColor(color);
            g2d.fillRect(center.x, center.y, size, size);
        }

        @Override
        public int getIconWidth() {
            return size;
        }

        @Override
        public int getIconHeight() {
            return size;
        }
    }
}
