package patternmatching;

import java.io.FileNotFoundException;
import java.io.PrintWriter;
import java.util.Arrays;
import java.util.function.Consumer;

public class Main {

    private  static class CustomObject {

        private int a;

        CustomObject(int a) {
            this.a = a;
        }

        @Override
        public String toString() {
            return "CustomObject{" +
                    "a=" + a +
                    '}';
        }
    }

    private static String formatObject(Object o) {
        if (o instanceof Integer) return "int " + o;
        if (o instanceof Byte) return "byte " + o;
        if (o instanceof Double) return "double " + o;
        if (o instanceof String) return "string " + o;
        return String.format("Object %s", o);
    }

    private static <T> long[] runNTimes(int numberOfRuns, T[] toTest, Consumer<T> code) {
        long runs[] = new long[numberOfRuns];
        for (int i = 0; i < numberOfRuns; i++) {
            long startTime = System.nanoTime();
            Arrays.stream(toTest).forEach(code);
            long endTime = System.nanoTime();
            long duration = (endTime - startTime);
            runs[i] = duration;
        }

        return runs;
    }

    public static void main(String[] args) {
        Object[] toTest = {"Hello", 1.2, new CustomObject(5)};

        final PatternMatcher<Object, String> pm = PatternMatcher
                .when(Integer.class::isInstance, i -> "int " + i)
                .orWhen(Byte.class::isInstance, b -> "byte " + b)
                .orWhen(Double.class::isInstance, d -> "double " + d)
                .orWhen(String.class::isInstance, s -> "String " + s)
                .otherwise(o -> String.format("Object %s", o));

        final int nRuns = 100000;
        final Statistics formatObjectStats = Statistics.from(runNTimes(nRuns, toTest, Main::formatObject));
        System.out.println(String.format("Time out of %d runs for formatObject(Object o) : %s", nRuns, formatObjectStats));
        final Statistics patternMatcherStats = Statistics.from(runNTimes(nRuns, toTest, pm::matches));
        System.out.println(String.format("Time out of %d runs for patternMatcher : %s", nRuns, patternMatcherStats));

        final PrintWriter writer;
        try {
            writer = new PrintWriter("/tmp/stats.csv");
            writer.println(formatObjectStats.asCSv());
            writer.println(patternMatcherStats.asCSv());
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
    }
}
