package messagebroker;

import messagebroker.handlers.AddHandlerMessageHandler;
import messagebroker.handlers.ListHandlersMessageHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.net.ServerSocket;
import java.net.Socket;

public class MessageBroker {
    private static final Logger LOGGER = LoggerFactory.getLogger(MessageBroker.class);

    private final ServerSocket serverSocket;
    private MessageHandlerFactory messageHandlerFactory;
    private final MessageHandlerContext context;

    private MessageBroker(int port) throws IOException {
        LOGGER.info("Initializing MessageBroker on port {}", port);
        this.serverSocket = new ServerSocket(port);
        context = new MessageHandlerContext(this);
        this.messageHandlerFactory = new MessageHandlerFactory(context);
    }

    private void run() {
        boolean running = true;
        while (running) {
            try {
                Socket accept = serverSocket.accept();

                ClientHandler handler = new ClientHandler(accept);
                new Thread(handler).start();

            } catch (IOException e) {
                LOGGER.error("Accept failed", e);
                running = false;
            }
        }
    }

    public void addHandler(String handlerName) throws ReflectiveOperationException {
        messageHandlerFactory.addHandler(handlerName);
    }

    public MessageHandler getHandler() {
        return messageHandlerFactory.getHandler();
    }

    private class ClientHandler implements Runnable {
        private final Socket socket;

        ClientHandler(Socket socket) {
            this.socket = socket;
        }

        @Override
        public void run() {
            try {
                BufferedReader reader = new BufferedReader(new InputStreamReader(socket.getInputStream()));
                String message = reader.readLine();
                LOGGER.info("Received message : {}", message);

                var outputStream = socket.getOutputStream();
                context.setOutputStream(outputStream);

                try {
                    messageHandlerFactory.getHandler().handleMessage(message);
                } catch (UnknownMessageFormatException e) {
                    LOGGER.warn("{}", e.getMessage());
                }

                socket.close();
            } catch (IOException e) {
                LOGGER.error("IOException on read");
            }
        }
    }

    public static void main(String[] args) throws IOException, ReflectiveOperationException {
        if (args.length != 1) {
            System.out.printf("usage : java %s <port>%n", MessageBroker.class.getName());
            System.exit(0);
        }

        int port = Integer.parseInt(args[0]);
        MessageBroker broker = new MessageBroker(port);
        broker.addHandler(AddHandlerMessageHandler.class.getCanonicalName());
        broker.addHandler(ListHandlersMessageHandler.class.getCanonicalName());
        broker.run();
    }
}
