package fr.gasser.reflection;

import javax.tools.JavaCompiler;
import javax.tools.ToolProvider;
import java.io.File;
import java.io.IOException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.net.URL;
import java.net.URLClassLoader;
import java.nio.charset.StandardCharsets;
import java.nio.file.Files;
import java.util.Arrays;

public class DynamicLoadDemo {
    /**
     * The source code to load dynamically
     */
    private static final String SOURCE =
            "package test;"
            + "import static fr.gasser.reflection.DynamicLoadDemo.*;"
            + "public class Test implements IDynamicLoad {"
            + "     static { System.out.println(\"Test\"); }"
            + "     @Override public String doSomething() {"
            + "         return \"Hello world!\";"
            + "     }"
            + "     public String doSomethingReflection(String arg) {"
            + "         return \"Hello reflection! \" + arg;"
            + "     }"
            + "}";

    /**
     * Represents the contract for the dynamically loaded class
     */
    public interface IDynamicLoad {
        String doSomething();
    }

    /**
     * @param args the command line arguments
     * @throws java.io.IOException
     * @throws java.lang.ReflectiveOperationException
     */
    public static void main(String[] args) throws IOException, ReflectiveOperationException {

        // Save source in .java file.
        final File root = Files.createTempDirectory(null).toFile();
        final File sourceFile = new File(root, "test/Test.java");
        if (!sourceFile.getParentFile().mkdirs()) {
            System.err.format("Cannot create %s%n", sourceFile.getCanonicalPath());
        }
        Files.write(sourceFile.toPath(), SOURCE.getBytes(StandardCharsets.UTF_8));

        // Compile source file.
        final JavaCompiler compiler = ToolProvider.getSystemJavaCompiler();
        compiler.run(null, null, null, sourceFile.getPath());

        // Load and instantiate compiled class.
        final URLClassLoader classLoader =
                URLClassLoader.newInstance(new URL[]{ root.toURI().toURL() });
        final Class<? extends IDynamicLoad> cls = Class.forName("test.Test", true, classLoader)
                .asSubclass(IDynamicLoad.class);
        final Constructor<? extends IDynamicLoad> ctor = cls.getConstructor();
        IDynamicLoad instance = ctor.newInstance();
        System.out.format(
                "Methods declared by the class : %s%n",
                Arrays.toString(cls.getDeclaredMethods())
        );

        // Invoke methods of the contract IDynamicLoad
        System.out.println(instance.doSomething());

        // Invoke methods using the reflection API
        Method method = cls.getDeclaredMethod("doSomethingReflection", String.class);
        System.out.format("invoking %s.doSomethingReflection()%n", cls.getName());
        System.out.println(method.invoke(instance, "With an argument"));
    }
}
